/*
 *  Copyright (C) 2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
    MCU_SPI0_D0 is your COPI(Controller Out Peripheral In). This should be connected to ADS8166 SDI.
    MCU_SPI0_D1 is your CIPO(Controller In Peripheral Out). This should be connected to ADS8166 SDO.
    MCU_SPI0_CS1 is your chip select and should be connected to ADS8166 CS line.
    MCU_SPI0_CLK should be connected to ADS8166 CLK line.
*/

#include <kernel/dpl/DebugP.h>
#include <kernel/dpl/ClockP.h>
#include "ti_drivers_config.h"
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"
#include <drivers/gpio.h>
#include <kernel/dpl/AddrTranslateP.h>
#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <kernel/dpl/ClockP.h>	
#include <kernel/dpl/DebugP.h>
#include <kernel/dpl/SemaphoreP.h>
#include <kernel/dpl/TaskP.h>
#include <drivers/ipc_notify.h>
#include <drivers/ipc_rpmsg.h>
#include <drivers/soc.h>
#include "ti_drivers_open_close.h"
#include "ti_drivers_config.h"
#include "ti_board_open_close.h"
#include "ipc_fw_version.h"
#include "FreeRTOS.h"
#include "task.h"

#define APP_MCSPI_MSGSIZE                   (1U)

uint32_t gMcspiTxBuffer[APP_MCSPI_MSGSIZE];
uint32_t gMcspiRxBuffer[APP_MCSPI_MSGSIZE];

//#include <kernel/dpl/threads.h>
//#include<kernel/dpl/pthread.h>
#define APP_MCSPI_MSGSIZE                   (1U)



#define IPC_RPMESSAGE_SERVICE_PING        "ti.ipc4.ping-pong"
#define IPC_RPMESSAGE_ENDPT_PING          (13U)

/* This is used to run the echo test with user space kernel */
#define IPC_RPMESSAGE_SERVICE_CHRDEV      "rpmsg_chrdev"
#define IPC_RPMESSAGE_ENDPT_CHRDEV_PING   (14U)

/* Use by this to receive ACK messages that it sends to other RTOS cores */
#define IPC_RPMESSAGE_RNDPT_ACK_REPLY     (11U)

/* Maximum size that message can have in this example
 * RPMsg maximum size is 512 bytes in linux including the header of 16 bytes.
 * Message payload size without the header is 512 - 16 = 496
 */
#define IPC_RPMESSAGE_MAX_MSG_SIZE        (496u)

/*
 * Number of RP Message ping "servers" we will start,
 * - one for ping messages for linux kernel "sample ping" client
 * - and another for ping messages from linux "user space" client using "rpmsg char"
 */
#define IPC_RPMESSAGE_NUM_RECV_TASKS         (2u)

/* RPMessage object used to recvice messages */
RPMessage_Object gIpcRecvMsgObject[IPC_RPMESSAGE_NUM_RECV_TASKS];

/* RPMessage object used to send messages to other non-Linux remote cores */
RPMessage_Object gIpcAckReplyMsgObject;

/* Task priority, stack, stack size and task objects, these MUST be global's */
#define IPC_RPMESSAGE_TASK_PRI         (8U)
#define LPM_MCU_UART_WAKEUP_TASK_PRI   (8U)

#if defined (SOC_AM62AX)
#define IPC_RPMESSAGE_TASK_STACK_SIZE  (32*1024U)
#else
#define IPC_RPMESSAGE_TASK_STACK_SIZE  (8*1024U)
#endif
#define LPM_MCU_UART_WAKEUP_TASK_STACK_SIZE   (1024U)

uint8_t gIpcTaskStack[IPC_RPMESSAGE_NUM_RECV_TASKS][IPC_RPMESSAGE_TASK_STACK_SIZE] __attribute__((aligned(32)));
TaskP_Object gIpcTask[IPC_RPMESSAGE_NUM_RECV_TASKS];

uint8_t gLpmUartWakeupTaskStack[LPM_MCU_UART_WAKEUP_TASK_STACK_SIZE] __attribute__((aligned(32)));
TaskP_Object gLpmUartWakeupTask;

/* number of iterations of message exchange to do */
uint32_t gMsgEchoCount = 100000u;



#if defined (SOC_AM64X)
uint32_t gRemoteCoreId[] = {
    CSL_CORE_ID_R5FSS0_0,
    CSL_CORE_ID_R5FSS0_1,
    CSL_CORE_ID_R5FSS1_0,
    CSL_CORE_ID_R5FSS1_1,
    CSL_CORE_ID_M4FSS0_0,
    CSL_CORE_ID_MAX /* this value indicates the end of the array */
};
static uint8_t gIpcInitiatorCoreID = CSL_CORE_ID_R5FSS0_0;
#endif

#if defined (SOC_AM62X)
uint32_t gRemoteCoreId[] = {
    CSL_CORE_ID_M4FSS0_0,
    CSL_CORE_ID_R5FSS0_0,
    CSL_CORE_ID_MAX /* this value indicates the end of the array */
};
uint8_t gMcuCoreID = CSL_CORE_ID_M4FSS0_0;
static uint8_t gIpcInitiatorCoreID = CSL_CORE_ID_R5FSS0_0;
#endif

#if defined (SOC_AM62AX)
uint32_t gRemoteCoreId[] = {
    CSL_CORE_ID_MCU_R5FSS0_0,
    CSL_CORE_ID_R5FSS0_0,
    CSL_CORE_ID_C75SS0_0,
    CSL_CORE_ID_MAX /* this value indicates the end of the array */
};
uint8_t gMcuCoreID = CSL_CORE_ID_MCU_R5FSS0_0;
static uint8_t gIpcInitiatorCoreID = CSL_CORE_ID_R5FSS0_0;
#endif

#if defined (SOC_AM62PX)
uint32_t gRemoteCoreId[] = {
    CSL_CORE_ID_MCU_R5FSS0_0,
    CSL_CORE_ID_WKUP_R5FSS0_0,
    CSL_CORE_ID_MAX /* this value indicates the end of the array */
};
uint8_t gMcuCoreID = CSL_CORE_ID_MCU_R5FSS0_0;
static uint8_t gIpcInitiatorCoreID = CSL_CORE_ID_WKUP_R5FSS0_0;
#endif



volatile uint8_t gbShutdown = 0u;
volatile uint8_t gbShutdownRemotecoreID = 0u;
volatile uint8_t gIpcAckReplyMsgObjectPending = 0u;
volatile uint8_t gbSuspended = 0u;
volatile uint32_t gNumBytesRead = 0;
volatile uint8_t gRecvTaskExitCounter = 0;

SemaphoreP_Object gLpmResumeSem;
SemaphoreP_Object gLpmSuspendSem;


void ipc_rpmsg_send_messages()
{
    RPMessage_CreateParams createParams;
    uint32_t msg, i, numRemoteCores;
    uint64_t curTime;
    char msgBuf[IPC_RPMESSAGE_MAX_MSG_SIZE];
    int32_t status;
    uint16_t remoteCoreId, msgSize;
    uint32_t remoteCoreEndPt;

    RPMessage_CreateParams_init(&createParams);
    createParams.localEndPt = IPC_RPMESSAGE_RNDPT_ACK_REPLY;
    status = RPMessage_construct(&gIpcAckReplyMsgObject, &createParams);
    DebugP_assert(status==SystemP_SUCCESS);

    numRemoteCores = 0;
    for(i=0; gRemoteCoreId[i]!=CSL_CORE_ID_MAX; i++)
    {
        if(gRemoteCoreId[i] != IpcNotify_getSelfCoreId()) /* dont count self */
        {
            numRemoteCores++;
        }
    }

    DebugP_log("[IPC RPMSG ECHO] Message exchange started with RTOS cores !!!\r\n");

    curTime = ClockP_getTimeUsec();

    for(msg=0; msg<gMsgEchoCount; msg++)
    {
        snprintf(msgBuf, IPC_RPMESSAGE_MAX_MSG_SIZE-1, "%d", msg);
        msgBuf[IPC_RPMESSAGE_MAX_MSG_SIZE-1] = 0;
        msgSize = strlen(msgBuf) + 1; /* count the terminating char as well */

        /* send the same messages to all cores */
        for(i=0; gRemoteCoreId[i]!=CSL_CORE_ID_MAX; i++ )
        {
            if(gRemoteCoreId[i] != IpcNotify_getSelfCoreId()) /* dont send message to self */
            {
                status = RPMessage_send(
                    msgBuf, msgSize,
                    gRemoteCoreId[i], IPC_RPMESSAGE_ENDPT_CHRDEV_PING,
                    RPMessage_getLocalEndPt(&gIpcAckReplyMsgObject),
                    SystemP_WAIT_FOREVER);
                DebugP_assert(status==SystemP_SUCCESS);
            }
        }
        /* wait for response from all cores */
        for(i=0; gRemoteCoreId[i]!=CSL_CORE_ID_MAX; i++ )
        {
            if(gRemoteCoreId[i] != IpcNotify_getSelfCoreId()) /* dont send message to self */
            {
                /* set 'msgSize' to size of recv buffer,
                * after return `msgSize` contains actual size of valid data in recv buffer
                */
                msgSize = sizeof(msgBuf);

                gIpcAckReplyMsgObjectPending = 1;
                status = RPMessage_recv(&gIpcAckReplyMsgObject,
                    msgBuf, &msgSize,
                    &remoteCoreId, &remoteCoreEndPt,
                    SystemP_WAIT_FOREVER);
                if (gbShutdown == 1u)
                {
                    break;
                }
                DebugP_assert(status==SystemP_SUCCESS);
                gIpcAckReplyMsgObjectPending = 0;

            }
        }
        if (gbShutdown == 1u)
        {
            break;
        }
    }
    gIpcAckReplyMsgObjectPending = 0;

    curTime = ClockP_getTimeUsec() - curTime;

    DebugP_log("[IPC RPMSG ECHO] All echoed messages received by main core from %d remote cores !!!\r\n", numRemoteCores);
    DebugP_log("[IPC RPMSG ECHO] Messages sent to each core = %d \r\n", gMsgEchoCount);
    DebugP_log("[IPC RPMSG ECHO] Number of remote cores = %d \r\n", numRemoteCores);
    DebugP_log("[IPC RPMSG ECHO] Total execution time = %" PRId64 " usecs\r\n", curTime);
    DebugP_log("[IPC RPMSG ECHO] One way message latency = %" PRId32 " nsec\r\n",
        (uint32_t)(curTime*1000u/(gMsgEchoCount*numRemoteCores*2)));

    RPMessage_destruct(&gIpcAckReplyMsgObject);
}


void ipc_rp_mbox_callback(uint16_t remoteCoreId, uint16_t clientId, uint32_t msgValue, void *args)
{
    if (clientId == IPC_NOTIFY_CLIENT_ID_RP_MBOX)
    {
        if (msgValue == IPC_NOTIFY_RP_MBOX_SHUTDOWN) /* Shutdown request from the remotecore */
        {
            gbShutdown = 1u;
            gbShutdownRemotecoreID = remoteCoreId;
            RPMessage_unblock(&gIpcRecvMsgObject[0]);
            RPMessage_unblock(&gIpcRecvMsgObject[1]);

            if (gIpcAckReplyMsgObjectPending == 1u)
                RPMessage_unblock(&gIpcAckReplyMsgObject);
        }
        else if (msgValue == IPC_NOTIFY_RP_MBOX_SUSPEND_SYSTEM) /* Suspend request from Linux. This is send when suspending to MCU only LPM */
        {
            gbSuspended = 1u;
            DebugP_memLogWriterPause();
            SemaphoreP_post(&gLpmSuspendSem);
            IpcNotify_sendMsg(remoteCoreId, IPC_NOTIFY_CLIENT_ID_RP_MBOX, IPC_NOTIFY_RP_MBOX_SUSPEND_ACK, 1u);
        }
        else if (msgValue == IPC_NOTIFY_RP_MBOX_ECHO_REQUEST) /* This message is received after resuming from the MCU only LPM. */
        {
            gbSuspended = 0u;

            if (gNumBytesRead != 0u)
            {
                /* post this only for MCU UART Wakeup */
                SemaphoreP_post(&gLpmResumeSem);
            }
            DebugP_shmLogWriterResume();

            IpcNotify_sendMsg(remoteCoreId, IPC_NOTIFY_CLIENT_ID_RP_MBOX, IPC_NOTIFY_RP_MBOX_ECHO_REPLY, 1u);
        }
    }
}
void uart_echo_read_callback(UART_Handle handle, UART_Transaction *trans)
{
    if (UART_TRANSFER_STATUS_SUCCESS == trans->status)
    {
        gNumBytesRead = trans->count;
    }
}



void ipc_recv_task_main(void *args)
{
    int32_t status;
    char recvMsg[IPC_RPMESSAGE_MAX_MSG_SIZE+1]; /* +1 for NULL char in worst case */
    uint16_t recvMsgSize, remoteCoreId;
    uint32_t remoteCoreEndPt;
    RPMessage_Object *pRpmsgObj = (RPMessage_Object *)args;

    DebugP_log("[IPC RPMSG ECHO] Remote Core waiting for messages at end point %d ... !!!\r\n",
        RPMessage_getLocalEndPt(pRpmsgObj)
        );

    /* wait for messages forever in a loop */
    while(1)
    {
        /* set 'recvMsgSize' to size of recv buffer,
        * after return `recvMsgSize` contains actual size of valid data in recv buffer
        */
        recvMsgSize = IPC_RPMESSAGE_MAX_MSG_SIZE;
        status = RPMessage_recv(pRpmsgObj,
            recvMsg, &recvMsgSize,
            &remoteCoreId, &remoteCoreEndPt,
            SystemP_WAIT_FOREVER);

        if (gbShutdown == 1u)
        {
            break;
        }

        if (gbSuspended == 1u)
        {
            continue;
        }

        DebugP_assert(status==SystemP_SUCCESS);

        /* echo the same message string as reply */
        #if 0 /* not logging this so that this does not add to the latency of message exchange */
        recvMsg[recvMsgSize] = 0; /* add a NULL char at the end of message */
        DebugP_log("%s\r\n", recvMsg);
        #endif

        /* send ack to sender CPU at the sender end point */
        status = RPMessage_send(
            recvMsg, recvMsgSize,
            remoteCoreId, remoteCoreEndPt,
            RPMessage_getLocalEndPt(pRpmsgObj),
            SystemP_WAIT_FOREVER);
        DebugP_assert(status==SystemP_SUCCESS);
    }

    gRecvTaskExitCounter++;
    if (gRecvTaskExitCounter >= IPC_RPMESSAGE_NUM_RECV_TASKS)
    {
        /* Follow the sequence for gracefull shutdown for the last recv task */
        DebugP_log("[IPC RPMSG ECHO] Closing all drivers and going to WFI ... !!!\r\n");

        /* Close the drivers */
        Drivers_close();

        /* deinit system */
        System_deinit();

        /* ACK the suspend message */
        IpcNotify_sendMsg(gbShutdownRemotecoreID, IPC_NOTIFY_CLIENT_ID_RP_MBOX, IPC_NOTIFY_RP_MBOX_SHUTDOWN_ACK, 1u);
#if (__ARM_ARCH_PROFILE == 'R') ||  (__ARM_ARCH_PROFILE == 'M')
        /* For ARM R and M cores*/
        __asm__ __volatile__ ("wfi"   "\n\t": : : "memory");
#endif
#if defined(BUILD_C7X)
        asm("    IDLE");
#endif
    }
    vTaskDelete(NULL);
}

void ipc_rpmsg_create_recv_tasks()
{
    int32_t status;
    RPMessage_CreateParams createParams;
    TaskP_Params taskParams;

    RPMessage_CreateParams_init(&createParams);
    createParams.localEndPt = IPC_RPMESSAGE_ENDPT_PING;
    status = RPMessage_construct(&gIpcRecvMsgObject[0], &createParams);
    DebugP_assert(status==SystemP_SUCCESS);

    RPMessage_CreateParams_init(&createParams);
    createParams.localEndPt = IPC_RPMESSAGE_ENDPT_CHRDEV_PING;
    status = RPMessage_construct(&gIpcRecvMsgObject[1], &createParams);
    DebugP_assert(status==SystemP_SUCCESS);

    /* We need to "announce" to Linux client else Linux does not know a service exists on this CPU
     * This is not mandatory to do for RTOS clients
     */
    status = RPMessage_announce(CSL_CORE_ID_A53SS0_0, IPC_RPMESSAGE_ENDPT_PING, IPC_RPMESSAGE_SERVICE_PING);
    DebugP_assert(status==SystemP_SUCCESS);

    status = RPMessage_announce(CSL_CORE_ID_A53SS0_0, IPC_RPMESSAGE_ENDPT_CHRDEV_PING, IPC_RPMESSAGE_SERVICE_CHRDEV);
    DebugP_assert(status==SystemP_SUCCESS);

    /* Create the tasks which will handle the ping service */
    TaskP_Params_init(&taskParams);
    taskParams.name = "RPMESSAGE_PING";
    taskParams.stackSize = IPC_RPMESSAGE_TASK_STACK_SIZE;
    taskParams.stack = gIpcTaskStack[0];
    taskParams.priority = IPC_RPMESSAGE_TASK_PRI;
    /* we use the same task function for echo but pass the appropiate rpmsg handle to it, to echo messages */
    taskParams.args = &gIpcRecvMsgObject[0];
    taskParams.taskMain = ipc_recv_task_main;

    status = TaskP_construct(&gIpcTask[0], &taskParams);
    DebugP_assert(status == SystemP_SUCCESS);

    TaskP_Params_init(&taskParams);
    taskParams.name = "RPMESSAGE_CHAR_PING";
    taskParams.stackSize = IPC_RPMESSAGE_TASK_STACK_SIZE;
    taskParams.stack = gIpcTaskStack[1];
    taskParams.priority = IPC_RPMESSAGE_TASK_PRI;
    /* we use the same task function for echo but pass the appropiate rpmsg handle to it, to echo messages */
    taskParams.args = &gIpcRecvMsgObject[1];
    taskParams.taskMain = ipc_recv_task_main;

    status = TaskP_construct(&gIpcTask[1], &taskParams);
    DebugP_assert(status == SystemP_SUCCESS);
}

uint32_t gMcspiRxBuffer[APP_MCSPI_MSGSIZE];

//static void i2c_read_error_handler(int32_t status);

void empty_main_i2c(void *arg0)
{
    int32_t         status = SystemP_SUCCESS;
    DebugP_log("\nstatus: %c", status);
    I2C_Handle      i2cHandle;
    uint8_t buffertowrite[3];
    uint8_t buffertowrite2[2];
    uint8_t buffertowrite1[1];
    uint8_t readBuffer[1];
    //uint32_t    loopcnt = 10, delayMsec = 3000U;

    /* STEP 1:
     *
     * Opening drivers, I2C being one of them.
     */

    /* STEP 2:
     *
     * Initializes I2C Handle.
     */
    i2cHandle = gI2cHandle[CONFIG_I2C0];
    
    // Detect all I2C devices
    //detect_all_i2c_devices(i2cHandle);
    
     /* STEP 3:
     *
     * Suppose our usecase is to set PIN-01 and PIN-05 to Output Mode, then we need to
     * write to Configuration Registers more specifically Configuration Port 0.
     * Configuration Port 0 is at 0x0C and the default value of this port is b"11111111"
     * We need to set the 0th bit, 2nd bit and 5th to 0. So the value to write would be b"11011010" which is 0xDA.
     */

// write config - configure Port 0 as output and Port 1 as input
        buffertowrite[0] = 0x8C;
        buffertowrite[1] = 0x00;
        buffertowrite[2] = 0xFF;
        I2C_Transaction i2cTransaction;
        I2C_Transaction_init(&i2cTransaction);
        i2cTransaction.slaveAddress = 0x22;
        i2cTransaction.writeBuf = buffertowrite;
        i2cTransaction.writeCount = 3;
        status = I2C_transfer(i2cHandle, &i2cTransaction);
        DebugP_assert(SystemP_SUCCESS == status);
        DebugP_log("\nTask2 - Set port0 as output for power switch and port1 as input from octocoupler");


//read the port - if any of the 8 pins of port 1 are low then turn corresponding port0 pins high
	while(1){
    	buffertowrite1[0] = 0x01;
//      I2C_Transaction_init(&i2cTransaction);
//      i2cTransaction.slaveAddress = 0x22; // I2C slave address
        i2cTransaction.writeBuf = buffertowrite1;
        i2cTransaction.writeCount = 1; // Writing only the register address
        i2cTransaction.readBuf = readBuffer; // Buffer to store the read data
        i2cTransaction.readCount = 1; // Reading 1 byte
    // Perform the I2C read operation continously
    status = I2C_transfer(i2cHandle, &i2cTransaction);
    DebugP_assert(SystemP_SUCCESS == status);
        DebugP_log("\nTask2loop - Read value from register 0x01: 0x%02X\n", readBuffer[0]);
// write the negated value of input status register of port1 to output status register of port 0
        buffertowrite2[0] = 0x04;
        buffertowrite2[1] = ~readBuffer[0];
        i2cTransaction.writeBuf = buffertowrite2;
        i2cTransaction.writeCount = 2;
        status = I2C_transfer(i2cHandle, &i2cTransaction);
        DebugP_assert(SystemP_SUCCESS == status);
        DebugP_log("\nTask2loop - Data written in 0x04 is 0x%02X\n", buffertowrite2[1]);
    }
}
/*
static void i2c_read_error_handler(int32_t status)
{
    switch(status)
    {
        case I2C_STS_ERR:
            DebugP_logError("\nGeneric error occurred");
            break;
        case I2C_STS_ERR_TIMEOUT:
            DebugP_logError("\nTimeout error occurred");
            break;
        case I2C_STS_ERR_NO_ACK:
            DebugP_logError("\nNo acknowledgement received\n");
            break;
        case I2C_STS_ERR_ARBITRATION_LOST:
            DebugP_logError("\nSample Arbitration lost\n");
            break;
        case I2C_STS_ERR_ACCESS_ERROR:
            DebugP_logError("\nBus Access error occurred\n");
            break;
    }

    return;
}
*/

void gpio_led_blink_main(void *args)
{
    uint32_t    loopcnt = 5, delaySec = 1;
    uint32_t    gpioBaseAddr, pinNum;

    DebugP_log("Task3 - GPIO LED Blink Test Started ...\r\n");
    DebugP_log("LED will Blink for %d seconds ...\r\n", (loopcnt * delaySec * 2));

    /* Get address after translation translate */
    gpioBaseAddr = (uint32_t) AddrTranslateP_getLocalAddr(GPIO_LED_BASE_ADDR);
    pinNum       = GPIO_LED_PIN;
    DebugP_log("Task3 - pinNum:%d\r\n",pinNum);
    GPIO_setDirMode(gpioBaseAddr, pinNum, GPIO_LED_DIR);
    while(loopcnt > 0)
    {
        GPIO_pinWriteHigh(gpioBaseAddr, pinNum);
        DebugP_log("Task3loop - LED High!!\r\n");
        ClockP_sleep(delaySec);
        GPIO_pinWriteLow(gpioBaseAddr, pinNum);
        DebugP_log("Task3loop - LED low!!\r\n");
        ClockP_sleep(delaySec);

        loopcnt--;
    }

    DebugP_log("Task3Finish - GPIO LED Blink Test Passed!!\r\n");
    DebugP_log("Task3Finish - All tests have passed!!\r\n");
}

void *empty_main_spi(void *args)
{
    uint8_t            i;
    int32_t             transferOK;
    MCSPI_Transaction   spiTransaction;

    DebugP_log("Task1 - ADS8166 Interfacing with MCSPI ...\r\n");

    /* Memfill buffers */
    for(i = 0U; i < APP_MCSPI_MSGSIZE; i++)
    {
        gMcspiTxBuffer[i] = 0U;
        gMcspiRxBuffer[i] = 0U;
    }

    /* 
    dataToWrite buffer has the following commands:
    -> 0x0800AA - reg access 
    -> 0x08041C - disable internal ref 
    -> 0x081010 - Data output format set
    -> 0x081802 - offset callib 
    -> 0x081C00 - Device config manual default 
    -> 0x081D04 - selects analog input for manual mode
    */
    uint32_t dataToWrite[7] = {0x0800AA, 0x08041C, 0x081020, 0x081802, 0x081C00, 0x081D03};
    for(int i = 0; i < 7; i++){

        // intialize the tx buffer
        gMcspiTxBuffer[0] = dataToWrite[i];

        /* define the transfer parameters */
        MCSPI_Transaction_init(&spiTransaction);
        spiTransaction.channel  = gConfigMcspi0ChCfg[0].chNum;
        spiTransaction.dataSize = 32;
        spiTransaction.csDisable = TRUE;
        spiTransaction.count    = 1;
        spiTransaction.txBuf    = (void *)gMcspiTxBuffer;
        spiTransaction.rxBuf    = (void *)gMcspiRxBuffer;
        spiTransaction.args     = NULL;

        // initiate the MCSPI transfer
        transferOK = MCSPI_transfer(gMcspiHandle[CONFIG_MCSPI0], &spiTransaction);


        if((SystemP_SUCCESS != transferOK) ||
        (MCSPI_TRANSFER_COMPLETED != spiTransaction.status))
        {
            DebugP_log("Task1 - Data 0x%08X is not written!! \r\n", dataToWrite[i]);
        }
        else
        {
            DebugP_log("Task1 - Data 0x%08X written successfully!! \r\n", dataToWrite[i]);
        }
        
        DebugP_log("gMcspiRxBuffer = 0x%08X \r\n", gMcspiRxBuffer[0]);
    }


//send nop commands continously
    while(1){

        // intialize the tx buffer
        gMcspiTxBuffer[0] = 0x0;

        /* define the transfer parameters */
        MCSPI_Transaction_init(&spiTransaction);
        spiTransaction.channel  = gConfigMcspi0ChCfg[0].chNum;
        spiTransaction.dataSize = 32;
        spiTransaction.csDisable = TRUE;
        spiTransaction.count    = 1;
        spiTransaction.txBuf    = (void *)gMcspiTxBuffer;
        spiTransaction.rxBuf    = (void *)gMcspiRxBuffer;
        spiTransaction.args     = NULL;

        // initiate the MCSPI transfer
        transferOK = MCSPI_transfer(gMcspiHandle[CONFIG_MCSPI0], &spiTransaction);


        if((SystemP_SUCCESS != transferOK) ||
        (MCSPI_TRANSFER_COMPLETED != spiTransaction.status))
        {
            DebugP_log("Task1loop - Data 0x%08X is not written!! \r\n", gMcspiTxBuffer[0]);
        }
        else
        {
            DebugP_log("Task1loop - Data 0x%08X written successfully!! \r\n", gMcspiTxBuffer[0]);
        }
        
        DebugP_log("Task1loop - gMcspiRxBuffer = 0x%08X \r\n", gMcspiRxBuffer[0]);
    }


    return NULL;
}



void empty_main(void *args)
{
        int32_t status;

    DebugP_log("[IPC RPMSG ECHO] Version: %s (%s %s):  \r\n", IPC_FW_VERSION, __DATE__, __TIME__);

    /* This API MUST be called by applications when its ready to talk to Linux */
    status = RPMessage_waitForLinuxReady(SystemP_WAIT_FOREVER);
    DebugP_assert(status==SystemP_SUCCESS);

    /* Register a callback for the RP_MBOX messages from the Linux remoteproc driver*/
    IpcNotify_registerClient(IPC_NOTIFY_CLIENT_ID_RP_MBOX, &ipc_rp_mbox_callback, NULL); 
    

#if defined(ENABLE_MCU_ONLY_LPM)
    if( IpcNotify_getSelfCoreId() == gMcuCoreID )
    {
        /* create task to monitor MCU uart to wakeup main domain.  */
        lpm_create_wakeup_task();
    }
#endif

    ipc_rpmsg_create_recv_tasks();

    /* wait for all non-Linux cores to be ready, this ensure that when we send messages below
     * they wont be lost due to rpmsg end point not created at remote core
     */
    IpcNotify_syncAll(SystemP_WAIT_FOREVER);

    /* Due to below "if" condition only one non-Linux core sends messages to all other non-Linux Cores
     * This is done mainly to show deterministic latency measurement
     */
    if( IpcNotify_getSelfCoreId() == gIpcInitiatorCoreID )
    {
        ipc_rpmsg_send_messages();
    }


    DebugP_log("All tests have passed!!\r\n");
}
